#!ruby
#**********************************************************************
# ruby correctpoint.rb YEAR CSVFILE
#----------------------------------------------------------------------
# 4大会目の選手の持ち点初期値を補正する
# rating.rb から呼び出しされる
# 入力
#   YEAR             集計対象の西暦年(yyyy)
#   CSVFILE          集計済みの最終の大会結果CSVファイル
#   FILE_MEIBO       成績名簿マスター
#   game/            対戦記録(gameファイル)の履歴
# 出力
#   FILE_PT_MEIBO    持ち点初期値補正された成績名簿マスター候補
#----------------------------------------------------------------------

require 'fileutils'
require './def_rating_files'
require './def_rating_meibo'

###################################################################
# 定数の定義

# findstrで抽出した順序付き対戦記録のリストのオフセット
GAME_NAME = 0		# 順序付き対戦記録のファイル名
GAME_WIN  = 1		# 勝者の選手コード
GAME_LOSE = 2		# 敗者の選手コード
GAME_ABS  = 3		# 棄権(Absence)

###################################################################
# グローバル変数の定義

# 成績名簿マスターのリスト
$meibo_db = []					# meibo_mochitenでのデータ検索用
$meibo_db_length = 0		# meibo_dbのデータ行数


###################################################################
# メソッド定義 pname, pt = meibo_mochiten(pcode)
# 選手コードに対応する選手名、持ち点を得る。
# 入力：
#  pcode  : 選手コード(String)
# 戻り値：
#  pname  : 選手の名前(String)
#  pt     : 選手の持ち点(Integer)

	def meibo_mochiten(pcode)
		i = 0
		while i < $meibo_db.length
			if pcode == $meibo_db[i][MB_CODE]
				pname = $meibo_db[i][MB_NAME]
				pt = $meibo_db[i][MB_PT]
				return pname, pt
			end
			i += 1
		end
		return "", 0
	end


###################################################################
# メソッド定義 pt = init_mochiten(pcode = "", pname = "", ppoint, csvfile)
# 4大会目の選手の持ち点初期値(補正あり)を算出する。
# 入力：
#  pcode   : 選手コード(String)
#  pname   : 選手の名前(String)
#  ppoint  : 補正前の選手の持ち点(Integer)
#  csvfile : 大会結果CSVファイル(String)
# 戻り値：
#  pt      : 補正後の選手の持ち点(Integer)

	def init_mochiten(pcode = "", pname = "", ppoint, csvfile)
		# 選手コードの対戦記録を抽出する
		cmd = "findstr #{pcode} game\\game-*.txt" + " > " + FILE_PT_GREP_GAME
		system(cmd)
		game = []
		f = File.open(FILE_PT_GREP_GAME, "r:CP932")
		while s = f.gets
			game.push s
		end
		f.close
		game.sort!.reverse!

		# gameディレクトリに集計対象日以降のgameファイルがないことを確認する
		game_date = game[0].slice(10,8)
		unless game_date == csvfile.slice(0,8)
			STDERR.puts "gameディレクトリのファイルが指定の大会開催日と一致しません。(#{game_date})"
			STDERR.puts ""
			exit 1
		end

		# 最近出場した4大会の対戦記録を切り出す
		i = 0
		tname0 = ""
		game_last4 = []

		game.each { |s|
			a = s.split(/\t/)
			tname1 = a[GAME_NAME].sub(/:[0-9]+/, "")
			if tname1 != tname0
				tname0 = tname1
				i += 1
				if i > 4
					break
				end
				game_last4.push s
			else
				game_last4.push s
			end
		}
		game_last4.sort!

		# 最近出場した4大会の負けた相手との持ち点差を評価する
		overpt_cnt = 0
		lose_cnt = 0
		lose_list = []
		tname0 = ""
		judgemark = ""
		aite_name = ""
		aite_point = 0
		change_pt = ppoint

		game_last4.each { |s|
			s.chomp!
			a = s.split(/\t/)
				aite_code = a[GAME_WIN].to_s
				aite_name, aite_point = meibo_mochiten(aite_code)
			if a.length == 3 && a[GAME_LOSE] == pcode
				if ppoint.to_i <= aite_point.to_i + 50
					judgemark = "○"
				else
					judgemark = "×"
					overpt_cnt += 1
				end
				lose_list.push [ lose_cnt, pcode, pname, ppoint.to_s, aite_code, aite_name, aite_point.to_s, judgemark ]
				lose_cnt += 1
			elsif a.length == 4 && a[GAME_LOSE] == pcode
				aite_name = aite_name + "(OFFER)"
				aite_point = "-"
				judgemark = "－"
				lose_list.push [ lose_cnt, pcode, pname, ppoint.to_s, aite_code, aite_name, aite_point.to_s, judgemark ]
				lose_cnt += 1
			end
		}

		# 持ち点初期値の補正判断内容をファイル出力して見える化する
		i = 0
		f1 = File.open(FILE_PT_HOSEI, "w")
		f2 = File.open(FILE_PT_HOSEILOG, "a")
		while i < 4
			pname      = lose_list[i][2]
			ppoint     = lose_list[i][3]
			aite_name  = lose_list[i][5]
			aite_point = lose_list[i][6]
			judgemark  = lose_list[i][7]
			if aite_name =~ /OFFER/
				f1.puts "#{pname}(持ち点#{ppoint})\t#{aite_name}\t#{judgemark}"
				f2.puts "#{pname}(持ち点#{ppoint})\t#{aite_name}\t#{judgemark}"
			else
				f1.puts "#{pname}(持ち点#{ppoint})\t#{aite_name}(持ち点#{aite_point})\t#{judgemark}"
				f2.puts "#{pname}(持ち点#{ppoint})\t#{aite_name}(持ち点#{aite_point})\t#{judgemark}"
			end
			i += 1
		end

		# 50点差超過が2回以上の場合は持ち点初期値を補正する
		if overpt_cnt >= 2
			pt0 = lose_list[0][6]
			pt1 = lose_list[1][6]
			pt2 = lose_list[2][6]
			pt3 = lose_list[3][6]
			aite_point_list = [ pt0, pt1, pt2, pt3 ]
			aite_point_list.sort!
			i = 0
			while aite_point_list[i] == "-"
				i += 1
			end
			i += 1
			if i < 4
				change_pt = aite_point_list[i]
			end
			f1.puts "change #{ppoint} ⇒ #{change_pt}"
			f2.puts "change #{ppoint} ⇒ #{change_pt}"
			STDERR.puts "#{pname} change #{ppoint} ⇒ #{change_pt}"
		end
		f1.close
		f2.close

		return change_pt
	end


#######################################################################
# メインプログラム

#######################################################################
## 引数チェック
## p1: year     ex) 2020
## p2: csvfile  ex) 20200126-007-大阪.csv

errmes = "rating.rb から実行してください。"

argc = ARGV.count
if argc != 2
	STDERR.puts errmes
	STDERR.puts ""
	exit 1
end

year = ARGV[0]
if year !~ /^[0-9]+$/
	STDERR.puts errmes
	STDERR.puts ""
	exit 1
end

csvfile = ARGV[1]
if csvfile !~ /^[0-9]+-[0-9]+-.+\.[cC][sS][vV]$/
	STDERR.puts errmes
	STDERR.puts ""
	exit 1
end


# ツールディレクトリをカレントにする
wkdir = RatingFiles.new(year)
tooldir = wkdir.rating_tooldir
Dir.chdir(tooldir)


#######################################################################
# 4大会目の選手を成績名簿マスターから抽出する
# (入力) csvfile     集計済みの最終の大会結果CSVファイル
#        meibo       成績名簿マスター
# (出力) player4th   成績名簿マスターから4大会目の選手を抽出した名簿

player4th = []
player4th_list = []

f = File.open(FILE_MEIBO, "r")
meibo = f.read
f.close

$meibo_db_length = 0
playernum = 0
date = wkdir.date_ymd_taikaicsv(csvfile)

# 対象の4大会目の選手を抽出してリストを作成する
f = File.open(FILE_PT_PLAYER4TH, "w")
meibo.each_line { |s|
	s.chomp!
	a = s.split(/\t/)
	$meibo_db.push a
	$meibo_db_length += 1
	if a[MB_TOUR].to_i == 4 && a[MB_DAY] == date
		f.printf("%s\n",s)
		playernum += 1
	end
}
f.close

if playernum == 0
	STDERR.puts "今回は４大会目の選手はいません。"
	exit 2
end

# 対象の4大会目の選手のリスト（持ち点の付与なし）
f = File.open(FILE_PT_PLAYER4TH, "r")
player4th = f.read
f.close

player4th.each_line { |s|
	s.chomp!
	player4th_list.push s
}
player4th_list.sort!

f = File.open(FILE_PT_HOSEILOG, "a")
f.puts ""
f.puts "#{csvfile}"
f.close

# 対象の4大会目の選手のリストを更新する（持ち点を付与する）
f = File.open(FILE_PT_PLAYER4TH, "w")
player4th_list.each { |s|
	a = s.split(/\t/)
	pt = init_mochiten(a[MB_CODE], a[MB_NAME], a[MB_PT].to_i, csvfile).to_s
	f.printf("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", \
		a[MB_CODE],a[MB_NAME],a[MB_KAI],a[MB_TOUR],a[MB_GAME],a[MB_WIN],a[MB_LOS],a[MB_NOP],pt,a[MB_DAY])
}
f.close

# 対象の4大会目の選手の持ち点を更新した新しい成績名簿マスター候補を作成する
fp = File.open(FILE_PT_PLAYER4TH, "r")
fm = File.open(FILE_MEIBO, "r")
f = File.open(FILE_PT_MEIBO, "w")

# タイトル行をコピーする
sp = fm.gets
f.print sp

while sp = fp.gets
	sp.chomp!
	a = sp.split(/\t/)
	while sm = fm.gets
		sm.chomp!
		b = sm.split(/\t/)
		if a[MB_CODE] == b[MB_CODE]
			f.puts sp
			break
		else
			f.puts sm
		end
	end
end


# 成績名簿マスターの残りを出力する
while sm = fm.gets
	f.print sm
end
fp.close
fm.close
f.close

