#!ruby
#**********************************************************************
# ruby rating.rb YEAR [off]
#----------------------------------------------------------------------
# 競技かるたレーティングシステム(Ruby版) メインプログラム
# 集計対象の西暦年ディレクトリに保管されている大会結果CSVファイルを
# rating_sub.rb に渡すデータ制御を行って、レーティング集計並びに
# ランキング作成を行う。
# また同一大会開催日ごとに4大会目の選手の持ち点初期値補正を行う。
# 入力：
#   YEAR   集計対象の西暦年(yyyy)
#   on     集計時のランキング画面出力オンのオプション
#          内部用ランキングTXTファイル作成と連動する
# 戻り値:
#   0:集計完了、1:エラー、2:集計対象なし
#----------------------------------------------------------------------
# 使用例：rating.batから実行される。
#   ruby rating.rb 2020
#   ruby rating.rb 2020 on
#----------------------------------------------------------------------

require './def_rating_files'


#######################################################################
# エラー終了処理

	def exit_error(errcode = "", csvfile = "")
		case errcode
			when "Unknown"
				if FileTest.exist?(FILE_ERROR) == true && FileTest.size(FILE_ERROR) > 0
					err = File.open(FILE_ERROR, "r")
					f = File.open(FILE_UPDATEMEMO, "a")
					f.puts ""
					f.puts csvfile
					f.print err.read
					err.close
					f.close
				end
				exit 1

			when "MismatchYear"
				puts "#{csvfile} の西暦年が指定の西暦年と一致しません。"
				puts ""
				exit 1
		end
	end


#######################################################################
# メインプログラム

#######################################################################
# 引数チェック
# ruby rating.rb YEAR [on]
# p1: YEAR, p2: on

argc = ARGV.count
year = ARGV[0]

unless (argc == 1 || argc == 2) && year =~ /^20[0-9][0-9]$/
	puts "レーティング集計ツール rating.rb の使い方"
	puts "コマンド形式：ruby rating.rb YEAR [on]"
	puts "YEAR：大会結果CSVファイル(yyyymmdd-nnn-name.csv)のある西暦年ディレクトリ"
	puts "on   : 集計時のランキング画面出力オンのオプション"
	puts ""
	puts "入力例：ruby rating.rb 2020"
	puts "入力例：ruby rating.rb 2020 on"
	exit 2
end

p2 = ARGV[1]
if p2 =~ /^[oO][nN]$/
	rank_display_on = 1		# 画面出力する
else
	rank_display_on = 0		# 画面出力しない
end


#######################################################################
# レーティング集計ツールの初期設定

# 使用ディレクトリの準備を行う
wkdir = RatingFiles.new(year)
tooldir = wkdir.rating_tooldir
Dir.chdir(tooldir)

# YDBリストの準備を行う
#（集計済み大会結果CSVファイルのリスト：昇順）
wkdir.make_ydb

# YDBリストの最終登録大会のシステム整合性確認を行う
wkdir.integrity_last_ydb

# CSVリストの準備を行う
#（集計対象西暦年の大会結果CSVファイルのリスト：昇順）
csvlist = []
csvlist = wkdir.make_csvlist
taikaicsv = ""
taikaicsv_end = taikaicsv


#######################################################################
# ランキングを作成する
# 大会結果CSVファイルをrating_sub.rbに1つずつ与えて集計を行い、
# ランキングを作成する。
# 中断が発生しなければ、同一大会開催日の大会を連続して集計できる。
# （持ち点初期値補正のため、複数の大会開催日の集計は不可）
# 次の場合に集計を中断し、exitする。
#   1: エラーが発生した場合、2:集計対象がない場合
# 不明な選手がいる場合は FILE_ERROR に出力してエラー終了する。
# この場合、大会結果CSVファイルや成績名簿マスター(FILE_MEIBO)を
# 見直し、データ修正して対応する。
#（改姓、所属会変更、異体字や表示のゆれ、記載誤り、新しい選手など）

flag_NotFoundCSV = 0
flag_MultiDate = 0
taikainum = 0
taikaidate = ""
multi_date_mes = ""

csvlist.each { |t|
	taikaicsv = t.chomp
	flag_NotFoundCSV = 1
	if wkdir.exist_taikai_in_ydb(taikaicsv) == 0
		# 大会開催日が同じかを確認する
		if taikainum == 0
			taikaidate = taikaicsv[0,8]
			unless year == taikaidate.slice(0,4)
				exit_error("MismatchYear", taikaicsv)
				break
			end
		elsif taikaidate != taikaicsv[0,8]
			flag_MultiDate = 1
			multi_date_mes = "Next. #{taikaicsv}"
			break
		end
		taikainum += 1

		# rating_sub.rbの実行
		if rank_display_on == 1
			cmd = "ruby rating_sub.rb " + year + " " + taikaicsv + " -rank_display_on"
		else
			cmd = "ruby rating_sub.rb " + year + " " + taikaicsv
		end
		ret = system(cmd)
		if ret == true
			taikaicsv_end = taikaicsv
		else
			exit_error("Unknown", taikaicsv)
			break
		end
	end
}

if flag_NotFoundCSV == 0
	puts "大会結果CSVファイルが見つかりませんでした。"
	puts ""
	exit 2
elsif taikainum == 0
	puts "すでに処理が完了しています。"
	puts ""
	exit 2
end


#######################################################################
# 4大会目の選手の持ち点初期値を補正する
# 入力
#   YEAR      集計対象の西暦年(yyyy)
#   CSVFILE   集計済みの最終の大会結果CSVファイル
# 出力
#   FILE_PT_MEIBO   持ち点初期値を補正した成績名簿マスター候補
#   成績名簿マスター候補をコピーして次のファイルを作成/更新する。
#     FILE_MEIBO      成績名簿マスター
#     FILE_MEIBO_BAK  成績名簿マスターの一時バックアップ
#     meibo/          成績名簿マスターの保存用

puts "初期値補正を行います。"
cmd = "ruby correctpoint.rb " + year + " " + taikaicsv_end
ret = system(cmd)

if FileTest.exist?(FILE_PT_MEIBO) == true && File.size?(FILE_PT_MEIBO) > 0
	wkdir.make_hosei_meibo(taikaicsv_end)
	puts "Final.   持ち点初期値の補正  -- OK"
	puts ""
end

# 持ち点初期化補正後の一時作業ファイルを削除する
wkdir.delete_tempfile_after_hosei


#######################################################################
# 複数の大会開催日の大会結果CSVファイルがあった場合に
# メッセージを表示して終了する。

if flag_MultiDate == 1
	puts "複数の大会開催日の大会結果CSVファイルがあります。"
	puts multi_date_mes
	puts ""
end
exit 0


